@@ -4,13 +4,25 @@ import logging
 import re
 import sys
 from contextvars import ContextVar
-from typing import Any, Callable, Dict, Union, cast
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    List,
+    Literal,
+    Optional,
+    Type,
+    TypeVar,
+    Union,
+    cast,
+    overload,
+)
 
-from omegaconf import DictConfig, MissingMandatoryValue, OmegaConf
+from omegaconf import Container, DictConfig, MissingMandatoryValue, OmegaConf
 
-from .application import Application, _log
-from .arg_parser import ArgumentError, ExitApplication
-from .type_resolvers import convert_to_type
+from .internal import Application
+from .internal.arg_parser import ArgumentError, ExitApplication
+from .internal.type_resolvers import convert_to_type
 
 __doc__ = """AlphaConf
 
@@ -35,47 +47,133 @@ configuration load and logging setup.
 """A list of functions which given a key indicate whether it's a secret"""
 SECRET_MASKS = [
     # mask if contains a kind of secret and it's not in a file
-    re.compile(r'.*(key|password|secret)s?(?!_file)(_|$)').match,
+    re.compile(r'.*(key|password|secret)s?(?!_file)(_|$)|^private(_key|$)').match,
 ]
 
 #######################################
 # APPLICATION CONTEXT
 
-"""The application context"""
-application: ContextVar[Application] = ContextVar('application')
 """The current configuration"""
 configuration: ContextVar[DictConfig] = ContextVar('configuration', default=OmegaConf.create())
 """Additional helpers for the application"""
 _helpers: ContextVar[Dict[str, str]] = ContextVar('configuration_helpers', default={})
 
+T = TypeVar('T')
 
-def select(container, key: str, type=None, *, default=None) -> Any:
-    """Select a configuration item from the container"""
-    if isinstance(container, DictConfig):
-        conf = container
+
+@overload
+def select(
+    container: Any,
+    key: str,
+    type: Type[T],
+    *,
+    default: Optional[T] = None,
+    required: Literal[True],
+) -> T:
+    ...
+
+
+@overload
+def select(
+    container: Any,
+    key: str,
+    type: Type[T],
+    *,
+    default: Optional[T] = None,
+    required: bool = False,
+) -> Optional[T]:
+    ...
+
+
+@overload
+def select(
+    container: Any,
+    key: str,
+    type: Union[str, Type[T], None] = None,
+    *,
+    default: Any = None,
+    required: bool = False,
+) -> Any:
+    ...
+
+
+def select(container: Any, key: str, type=None, *, default=None, required: bool = False) -> Any:
+    """Select a configuration item from the container
+
+    :param container: The container to select from (Container, dict, etc.)
+    :param key: The selection key
+    :param type: The type of the object to return
+    :param default: The default value is selected value is None
+    :param required: Raise MissingMandatoryValue if the selected value and default are None
+    :return: The selected value in the container
+    """
+    c: Any
+    # make sure we have a container and select from it
+    if isinstance(container, Container):
+        c = container
     else:
-        conf = OmegaConf.create(container)
-    if key:
-        c = OmegaConf.select(conf, key, throw_on_missing=True)
-    else:
-        c = conf
-    if isinstance(c, DictConfig):
-        c = OmegaConf.to_object(c)
-    elif c is None:
+        c = OmegaConf.create(container)
+    c = OmegaConf.select(c, key, throw_on_missing=required)
+    # handle empty result
+    if c is None:
+        if default is None and required:
+            raise MissingMandatoryValue("Key not found: %s" % key)
         return default
-    if type and c is not None:
+    # check the returned type and convert when necessary
+    if type is not None and isinstance(c, type):
+        return c
+    if isinstance(c, Container):
+        c = OmegaConf.to_object(c)
+    if type is not None:
         c = convert_to_type(c, type)
     return c
 
 
-def get(config_key: str, type=None, *, default=None) -> Any:
+@overload
+def get(
+    config_key: str,
+    type: Type[T],
+    *,
+    default: Optional[T] = None,
+    required: Literal[True],
+) -> T:
+    ...
+
+
+@overload
+def get(
+    config_key: str,
+    type: Type[T],
+    *,
+    default: Optional[T] = None,
+    required: bool = False,
+) -> Optional[T]:
+    ...
+
+
+@overload
+def get(
+    config_key: str,
+    type: Union[str, Type[T], None] = None,
+    *,
+    default: Any = None,
+    required: bool = False,
+) -> Any:
+    ...
+
+
+def get(config_key: str, type=None, *, default=None, required: bool = False) -> Any:
     """Select a configuration item from the current configuration"""
-    return select(configuration.get(), config_key, type=type, default=default)
+    return select(configuration.get(), config_key, type=type, default=default, required=required)
 
 
 @contextlib.contextmanager
 def set(**kw):
-    """Update the configuration in a with block"""
+    """Update the configuration in a with block
+
+    with alphaconf.set(a=value):
+        assert alphaconf.get('a') == value
+    """
     if not kw:
         yield
         return
@@ -93,7 +191,6 @@ def set_application(app: Application, merge: bool = False):
     :param app: The application
     :param merge: Wether to merge the current configuration with the application (default false)
     """
-    application.set(app)
     config = app.configuration
     if merge:
         # merging 2 DictConfig objects
@@ -101,7 +198,14 @@ def set_application(app: Application, merge: bool = False):
     configuration.set(config)
 
 
-def run(main: Callable, arguments=True, *, should_exit=True, app: Application = None, **config):
+def run(
+    main: Callable[[], T],
+    arguments: Union[bool, List[str]] = True,
+    *,
+    should_exit: bool = True,
+    app: Application = None,
+    **config,
+) -> Optional[T]:
     """Run this application
 
     If an application is not given, a new one will be created with configuration properties
@@ -113,6 +217,8 @@ def run(main: Callable, arguments=True, *, should_exit=True, app: Application =
     :param config: Arguments passed to Application.__init__() and Application.setup_configuration()
     :return: The result of main
     """
+    from .internal import application_log as log
+
     if 'setup_logging' not in config:
         config['setup_logging'] = True
     if app is None:
@@ -121,44 +227,57 @@ def run(main: Callable, arguments=True, *, should_exit=True, app: Application =
             for k in ['name', 'version', 'description', 'short_description']
             if k in config
         }
+        # if we don't have a description, get it from the function's docs
+        if 'description' not in properties and main.__doc__:
+            description = main.__doc__.strip().split('\n', maxsplit=1)
+            if 'short_description' not in properties:
+                properties['short_description'] = description[0]
+            if len(description) > 1:
+                import textwrap
+
+                properties['description'] = description[0] + '\n' + textwrap.dedent(description[1])
+            else:
+                properties['description'] = properties['short_description']
         app = Application(**properties)
     try:
         app.setup_configuration(arguments, **config)
     except MissingMandatoryValue as e:
-        _log.error(e)
+        log.error(e)
         if should_exit:
             sys.exit(99)
         raise
     except ArgumentError as e:
-        _log.error(e)
+        log.error(e)
         if should_exit:
             sys.exit(2)
         raise
     except ExitApplication:
-        _log.debug('Normal application exit')
+        log.debug('Normal application exit')
         if should_exit:
             sys.exit()
-        return
+        return None
     context = contextvars.copy_context()
     try:
         return context.run(__run_application, app=app, main=main, exc_info=should_exit)
     except Exception:
         if should_exit:
-            _log.debug('Exit application')
+            log.debug('Exit application')
             sys.exit(1)
         raise
 
 
-def __run_application(app: Application, main: Callable, exc_info=True):
+def __run_application(app: Application, main: Callable[[], T], exc_info=True) -> T:
     """Set the application and execute main"""
     set_application(app)
     app_log = logging.getLogger()
     if get('testing', bool):
         app_log.info('Application testing (%s: %s)', app.name, main.__qualname__)
-        return None
+        return get('testing')
     # Run the application
     try:
         app_log.info('Application start (%s: %s)', app.name, main.__qualname__)
+        for missing_key in OmegaConf.missing_keys(configuration.get()):
+            app_log.warning('Missing configuration key: %s', missing_key)
         result = main()
         if result is None:
             app_log.info('Application end.')
