@@ -2,7 +2,16 @@ import invoke
 from omegaconf import OmegaConf
 
 from . import run as _application_run
-from .application import Application, arg_parser
+from .internal import Application, arg_parser
+
+__doc__ = """Invoke wrapper for an application
+
+Adding the following lines at the end of the file adds support for alphaconf
+to inject the configuration.
+
+    ns = Collection(...)
+    alphaconf.invoke.run(__name__, ns)
+"""
 
 
 class InvokeAction(arg_parser.Action):
@@ -19,7 +28,7 @@ class InvokeApplication(Application):
         self.namespace = namespace
         self.argument_parser.add_argument(
             InvokeAction,
-            metavar="invoke arguments",
+            metavar="-- invoke arguments",
             help="Rest is passed to invoke",
         )
 
